/*
   DocumentRef.java -- Reference to an indexed document. Keeps track of all information stored
   on the document, either by the dig or temporary search information.


   $Id: DocumentRef.java,v 1.1 1999/05/24 19:41:57 eg Exp $

   Copyright (c) 1995-1999 The ht://Dig Group
   Copyright (C) 1999, Edwin Glaser

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

package htsearch;

/**
 * DocumentRef -- Reference to an indexed document. Keeps track of all information stored
 *  on the document, either by the dig or temporary search information.
 */

public class DocumentRef 
{
  static boolean trace = false;

  static final int DOC_ID = 0;
  static final int DOC_TIME = 1;
  static final int DOC_ACCESSED = 2;
  static final int DOC_STATE = 3;
  static final int DOC_SIZE = 4;
  static final int DOC_LINKS = 5;
  static final int DOC_IMAGESIZE = 6;
  static final int DOC_HOPCOUNT = 7;
  static final int DOC_URL = 8;
  static final int DOC_HEAD = 9;
  static final int DOC_TITLE = 10;
  static final int DOC_DESCRIPTIONS = 11;
  static final int DOC_ANCHORS = 12;
  static final int DOC_EMAIL = 13;
  static final int DOC_NOTIFICATION = 14;
  static final int DOC_SUBJECT = 15;
  static final int DOC_STRING = 16;
  static final int DOC_METADSC = 17;
  static final int DOC_BACKLINKS = 18;
  static final int DOC_SIG = 19;

  static final int CHARSIZE_MARKER_BIT = 64;
  static final int SHORTSIZE_MARKER_BIT = 128;

  public int docID;
  // This is the URL of the document.
  public String docURL;
  // This is the time specified in the document's header
  // Usually that's the last modified time, for servers that return it.
  public long docTime;
  // Usually that's the last modified time = This is the time that the last retrieval occurred.;
  public long docAccessed;
  // This is the stored excerpt of the document, just text.
  public String docHead;
  // This is the document-specified description.
  // For HTML, that's the META description tag.
  public String docMetaDsc;
  // This is the title of the document.
  public String docTitle;
  // This is a list of Strings, the text of links pointing to this document.
  // (e.g. <a href="docURL">description</a>
  public String[] descriptions;
  // This is the state of the document--modified, normal, etc.
  public int docState;
  // This is the size of the original document.
  public int docSize;
  // This is a count of the links in the document (outgoing links).
  public int docLinks;
  // This is a count of the links to the document (incoming links).
  public int docBackLinks;
  // This is the size of the document when including images.
  public int docImageSize;
  // This is a list of the anchors in the document (i.e. <A NAME=...)
  public String[] docAnchors;
  // This is a count of the number of hops from start_urls to here.
  public int docHopCount;
  // This is a signature of the document. (e.g. md5sum, checksum...)
  // This is currently unused.
  public int docSig;
	
  //
  // The following values are for the email notification of expiration
  //

  // This is the email destination for htnotify.
  public String docEmail;
  // This is the date that htnotify should use as comparison.
  public String docNotification;
  // This is the subject of the email sent out by htnotify.
  public String docSubject;

  //
  // This is used for searching and is not stored in the database
  //
    
  // This is the current score of this document.
  public int docScore;
  // This is the nearest anchor for the search word.
  public int docAnchor;


  long getnum(int type, byte[] entry, int[] pos)
  {
	long ret = 0;
	if ((type & CHARSIZE_MARKER_BIT) != 0)
	{
		ret = (((long) entry[pos[0]]) & 0xff);
		if ( trace ) System.out.println("getnum : " + (((long) entry[pos[0]]) & 0xff));
		pos[0] = pos[0] + 1;
	}
	else if ((type & SHORTSIZE_MARKER_BIT) != 0)
	{
		ret = ((((long) entry[pos[0] + 1]) & 0xff) << 8)
		  + (((long) entry[pos[0]]) & 0xff);
		if ( trace ) System.out.println("getnum : " + (((long) entry[pos[0] + 1]) & 0xff) + " " + (((long) entry[pos[0]]) & 0xff));
		pos[0] = pos[0] + 2;
	}
	else
	{
		ret = ((((long) entry[pos[0] + 3]) & 0xff) << 24)
		  + ((((long) entry[pos[0] + 2]) & 0xff) << 16)
		  + ((((long) entry[pos[0] + 1]) & 0xff) << 8)
		  + (((long) entry[pos[0]]) & 0xff);
		if ( trace ) System.out.println("getnum : " + (((long) entry[pos[0] + 3]) & 0xff) + " " + (((long) entry[pos[0] + 2]) & 0xff) + " " + (((long) entry[pos[0] + 1]) & 0xff) + " " + (((long) entry[pos[0]]) & 0xff));
		if ( trace ) System.out.println("getnum : 0x" + Long.toHexString(ret) + " = " + Long.toString(ret));
		pos[0] = pos[0] + 4;
	}
	return ret;
  }

  String getstring(int type, byte[] entry, int[] pos)
  {
	int length = (int)getnum(type, entry, pos);
	String str = new String(entry, pos[0], length);
	pos[0] = pos[0] + length;
	return str;
  }

  String[] getlist(int type, byte[] entry, int[] pos)
  {
	int count = (int)getnum(type, entry, pos);
	String[] ret = new String [count];
	
	if ((type & (CHARSIZE_MARKER_BIT | SHORTSIZE_MARKER_BIT)) != 0 )
	{
		int length;
		for (int i = 0; i < count; i++)                                    
		{      
			int tmp = ((int)entry[pos[0]]) & 0xff;
			pos[0] = pos[0] + 1;
			if ( tmp < 0xff )
			{
				length = tmp;
			}
			else
			{
				length = (int)getnum(0, entry, pos);
			}
			ret[i] =  new String(entry, pos[0], length);
			pos[0] = pos[0] + length;
		}                            
	}                                                                
	else                                                             
	{                                                                
		int length;
		for (int i = 0; i < count; i++)                                    
		{      
			length = (int)getnum(0, entry, pos);
			ret[i] =  new String(entry, pos[0], length);
			pos[0] = pos[0] + length;
		}                            
	}
	return ret;
  }

  public void Deserialize(byte[] stream)
  {
	//Clear();

	int pos[] = new int[1];
	pos[0] = 0;
	while ( pos[0] < stream.length )
	{
		int x = ((int)stream[pos[0]]) & 0xff;
		pos[0] = pos[0] + 1;
		switch ( x & ~(CHARSIZE_MARKER_BIT | SHORTSIZE_MARKER_BIT) )
		{
        case DOC_ID:
		  docID = (int)getnum(x, stream, pos);
		  if ( trace ) System.out.println("docID : " + docID);
		  break;
        case DOC_TIME:
		  docTime = getnum(x, stream, pos);
		  if ( trace ) System.out.println("docTime : " + docTime);
		  break;
        case DOC_ACCESSED:
		  docAccessed = getnum(x, stream, pos);
		  if ( trace ) System.out.println("docAccessed : " + docAccessed);
		  break;
        case DOC_STATE:
		  docState = (int)getnum(x, stream, pos);
		  if ( trace ) System.out.println("docState : " + docState);
		  break;
        case DOC_SIZE:
		  docSize = (int)getnum(x, stream, pos);
		  if ( trace ) System.out.println("docSize : " + docSize);
		  break;
        case DOC_LINKS:
		  docLinks = (int)getnum(x, stream, pos);
		  if ( trace ) System.out.println("docLinks : " + docLinks);
		  break;
        case DOC_IMAGESIZE:
		  docImageSize = (int)getnum(x, stream, pos);
		  if ( trace ) System.out.println("docImageSize : " + docImageSize);
		  break;
        case DOC_HOPCOUNT:
		  docHopCount = (int)getnum(x, stream, pos);
		  if ( trace ) System.out.println("docHopCount : " + docHopCount);
		  break;
		case DOC_BACKLINKS:
		  docBackLinks = (int)getnum(x, stream, pos);
		  if ( trace ) System.out.println("docBackLinks : " + docBackLinks);
		  break;
		case DOC_SIG:
		  docSig = (int)getnum(x, stream, pos);
		  if ( trace ) System.out.println("docSig : " + docSig);
		  break;
        case DOC_URL:
		  docURL = HtURLCodec.instance().decode(getstring(x, stream, pos));
		  if ( trace ) System.out.println("docURL : " + docURL);
		  break;
        case DOC_HEAD:
		  docHead = getstring(x, stream, pos);
		  if ( trace ) System.out.println("docHead : " + docHead);
		  break;
		case DOC_METADSC:
		  docMetaDsc = getstring(x, stream, pos);
		  if ( trace ) System.out.println("docMetaDsc : " + docMetaDsc);
		  break;
        case DOC_TITLE:
		  docTitle = getstring(x, stream, pos);
		  if ( trace ) System.out.println("docTitle : " + docTitle);
		  break;
        case DOC_DESCRIPTIONS:
		  descriptions = getlist(x, stream, pos);
		  if ( trace ) System.out.println("descriptions : " + descriptions);
		  break;
        case DOC_ANCHORS:
		  docAnchors = getlist(x, stream, pos);
		  if ( trace ) System.out.println("docAnchors : " + docAnchors);
		  break;
        case DOC_EMAIL:
		  docEmail = getstring(x, stream, pos);
		  if ( trace ) System.out.println("docEmail : " + docEmail);
		  break;
        case DOC_NOTIFICATION:
		  docNotification = getstring(x, stream, pos);
		  if ( trace ) System.out.println("docNotification : " + docNotification);
		  break;
        case DOC_SUBJECT:
		  docSubject = getstring(x, stream, pos);
		  if ( trace ) System.out.println("docSubject : " + docSubject);
		  break;
		case DOC_STRING:
		  // This is just a debugging string. Ignore it.
		  break;
        default:
		  System.err.println("BAD TAG IN SERIALIZED DATA: " + (int)x);
		  return;
        }

	}
  }

  public DocumentRef()
  {
	
  }

  public DocumentRef(byte[] stream)
  {
	Deserialize(stream);	
  }

  public String toString()
  {
	return "DocumentRef(" + docURL + ", " + docSize + ", " + docScore + ", " + docAnchor + ")";
  }

} // DocumentRef
